@@ -311,7 +311,9 @@ class SerializerBase(object):
                 value["__class__"] = obj.__class__.__module__ + "." + obj.__class__.__name__
                 return value
             else:
-                raise errors.SerializeError("don't know how to serialize class " + str(obj.__class__) + " using serializer " + str(cls.__name__) + ". Give it vars() or an appropriate __getstate__")
+                raise errors.SerializeError("don't know how to serialize class " + str(obj.__class__) +
+                                            " using serializer " + str(cls.__name__) +
+                                            ". Give it vars() or an appropriate __getstate__")
 
     @classmethod
     def dict_to_class(cls, data):
@@ -354,6 +356,8 @@ class SerializerBase(object):
                 return JsonSerializer()
             elif classname == "Pyro4.util.MsgpackSerializer":
                 return MsgpackSerializer()
+            elif classname == "Pyro4.util.CloudpickleSerializer":
+                return CloudpickleSerializer()
             elif classname == "Pyro4.util.DillSerializer":
                 return DillSerializer()
         elif classname.startswith("Pyro4.errors."):
@@ -451,6 +455,40 @@ class PickleSerializer(SerializerBase):
         def copyreg_function(obj):
             return replacement_function(obj).__reduce__()
 
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
+        try:
+            copyreg.pickle(object_type, copyreg_function)
+        except TypeError:
+            pass
+
+
+class CloudpickleSerializer(SerializerBase):
+    """
+    A (de)serializer that wraps the Cloudpickle serialization protocol.
+    It can optionally compress the serialized data, and is thread safe.
+    """
+    serializer_id = 7  # never change this
+
+    def dumpsCall(self, obj, method, vargs, kwargs):
+        return cloudpickle.dumps((obj, method, vargs, kwargs), config.PICKLE_PROTOCOL_VERSION)
+
+    def dumps(self, data):
+        return cloudpickle.dumps(data, config.PICKLE_PROTOCOL_VERSION)
+
+    def loadsCall(self, data):
+        return cloudpickle.loads(data)
+
+    def loads(self, data):
+        return cloudpickle.loads(data)
+
+    @classmethod
+    def register_type_replacement(cls, object_type, replacement_function):
+        def copyreg_function(obj):
+            return replacement_function(obj).__reduce__()
+
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
         try:
             copyreg.pickle(object_type, copyreg_function)
         except TypeError:
@@ -481,6 +519,8 @@ class DillSerializer(SerializerBase):
         def copyreg_function(obj):
             return replacement_function(obj).__reduce__()
 
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
         try:
             copyreg.pickle(object_type, copyreg_function)
         except TypeError:
@@ -566,6 +606,8 @@ class SerpentSerializer(SerializerBase):
             else:
                 serpent_serializer._serialize(replaced, outputstream, indentlevel)
 
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
         serpent.register_class(object_type, custom_serializer)
 
     @classmethod
@@ -617,6 +659,8 @@ class JsonSerializer(SerializerBase):
 
     @classmethod
     def register_type_replacement(cls, object_type, replacement_function):
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
         cls.__type_replacements[object_type] = replacement_function
 
 
@@ -680,10 +724,12 @@ class MsgpackSerializer(SerializerBase):
             return datetime.datetime.fromtimestamp(struct.unpack("d", data)[0])
         if code == 0x33:
             return datetime.date.fromordinal(struct.unpack("l", data)[0])
-        raise errors.SerializeError("invalid ext code for msgpack: "+str(code))
+        raise errors.SerializeError("invalid ext code for msgpack: " + str(code))
 
     @classmethod
     def register_type_replacement(cls, object_type, replacement_function):
+        if object_type is type or not inspect.isclass(object_type):
+            raise ValueError("refusing to register replacement for a non-type or the type 'type' itself")
         cls.__type_replacements[object_type] = replacement_function
 
 
@@ -705,6 +751,7 @@ def get_serializer_by_id(sid):
     except KeyError:
         raise errors.SerializeError("no serializer available for id %d" % sid)
 
+
 # determine the serializers that are supported
 try:
     import cPickle as pickle
@@ -718,6 +765,13 @@ import marshal
 _ser = MarshalSerializer()
 _serializers["marshal"] = _ser
 _serializers_by_id[_ser.serializer_id] = _ser
+try:
+    import cloudpickle
+    _ser = CloudpickleSerializer()
+    _serializers["cloudpickle"] = _ser
+    _serializers_by_id[_ser.serializer_id] = _ser
+except ImportError:
+    pass
 try:
     import dill
     _ser = DillSerializer()
@@ -743,8 +797,8 @@ try:
     else:
         ver = serpent.__version__
     ver = tuple(map(int, ver.split(".")))
-    if ver < (1, 19):
-        raise RuntimeError("requires serpent 1.19 or better")
+    if ver < (1, 23):  # serpent 1.23 required
+        raise RuntimeError("requires serpent 1.23 or better")
     _ser = SerpentSerializer()
     _serializers["serpent"] = _ser
     _serializers_by_id[_ser.serializer_id] = _ser
@@ -809,6 +863,7 @@ def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
 
 __exposed_member_cache = {}
 
+
 def reset_exposed_members(obj, only_exposed=True, as_lists=False):
     """Delete any cached exposed members forcing recalculation on next request"""
     if not inspect.isclass(obj):
@@ -816,6 +871,7 @@ def reset_exposed_members(obj, only_exposed=True, as_lists=False):
     cache_key = (obj, only_exposed, as_lists)
     __exposed_member_cache.pop(cache_key, None)
 
+
 def get_exposed_members(obj, only_exposed=True, as_lists=False, use_cache=True):
     """
     Return public and exposed members of the given object's class.
@@ -902,7 +958,7 @@ def set_exposed_property_value(obj, propname, value, only_exposed=True):
 _private_dunder_methods = frozenset([
     "__init__", "__call__", "__new__", "__del__", "__repr__", "__unicode__",
     "__str__", "__format__", "__nonzero__", "__bool__", "__coerce__",
-    "__cmp__", "__eq__", "__ne__", "__hash__",
+    "__cmp__", "__eq__", "__ne__", "__hash__", "__ge__", "__gt__", "__le__", "__lt__",
     "__dir__", "__enter__", "__exit__", "__copy__", "__deepcopy__", "__sizeof__",
     "__getattr__", "__setattr__", "__hasattr__", "__getattribute__", "__delattr__",
     "__instancecheck__", "__subclasscheck__", "__getinitargs__", "__getnewargs__",
